#ifndef LSM_STR
#define LSM_STR

#include "lsm.h"

/**
 * Represents a string (or really any kind of data) with a known length. Data
 * with length 8 or less is stored inside the pointer, and does not allocate
 * additional memory.
 */
typedef struct lsm_str lsm_str;

/**
 * Allocate a new string struct of length 0.
 *
 * @param ptr pointer to store newly allocated pointer in
 */
lsm_error lsm_str_init_zero(lsm_str **ptr);

/**
 * Update an existing lsm_str so it now represents the new provided string. The
 * string pointer of the original object is free'd if needed.
 *
 * @param str lsm_str object to modify
 * @param s string to convert into lsm string; ownership is taken over
 */
void lsm_str_init_prealloc(lsm_str *str, char *s);

/**
 * Allocate and initialize a new lsm_str object
 *
 * @param ptr pointer to store newly allocated pointer
 * @param s string to convert into lsm string; ownership is taken over
 */
lsm_error lsm_str_init(lsm_str **ptr, char *s);

/**
 * Deallocate the existing internal string if needed and replace the lsm_str
 * with a string of length 0, wiping its contents.
 *
 * @param str string to wipe
 */
void lsm_str_zero(lsm_str *str);

/**
 * Deallocate the string and its internal char buffer if needed. Only call this
 * on heap-allocated strings.
 *
 * @param str string to dealloate
 */
void lsm_str_free(lsm_str *str);

/**
 * Return the length of the string.
 *
 * @param str string to return length for.
 */
uint64_t lsm_str_len(lsm_str *str);

#endif
